// 
// Joergs quick'n dirty hack to read & write JPEGs using the libjpeg library
// Based upon example.c
// 

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>

#include "jpeg.h"
#include "jpeglib.h"


unsigned char*
read_JPEG_file(char filename[200], unsigned int *width, unsigned int *height)
{
  unsigned char* image;
  struct jpeg_error_mgr jerr;
  struct jpeg_decompress_struct cinfo;  // This struct contains the JPEG decompression
                                        // parameters and pointers to working space 
                                        // (which is allocated as needed by the JPEG 
                                        // library). 
  FILE * infile;			// source file
  JSAMPARRAY buffer;			// Output row buffer 
  int row_stride;			// physical row width in output buffer


  //printf("    _______________________            \n");
  //printf("+--< Start: read_JPEG_file >-----------\n");
  //printf("| \n");
  //printf("| file:        %s\n", filename);
  //printf("|--------------------------\n");

  if ((infile = fopen(filename, "rb")) == NULL) {
    fprintf(stderr, "can't open %s\n", filename);
    return 0;
  }

  cinfo.err = jpeg_std_error(&jerr);	// set up the normal JPEG error routines
  jpeg_create_decompress(&cinfo);	// initialize the JPEG decompression object.
  jpeg_stdio_src(&cinfo, infile);	// specify data source
  (void) jpeg_read_header(&cinfo, TRUE);//read file parameters

  *width  = cinfo.image_width;
  *height = cinfo.image_height;
  //printf("| Detected resolution: %i x %i\n", *width, *height);

  (void) jpeg_start_decompress(&cinfo);	// Start decompressor

  // JSAMPLEs per row in output buffer
  row_stride = cinfo.output_width * cinfo.output_components;

  // create an image buffer
  image = malloc( cinfo.output_width * cinfo.output_height * cinfo.output_components * sizeof(unsigned char) );
  if(image==NULL)
    printf("init: out of memory\n");
  //else
  //  printf("| image buffer allocated: %x\n", image);

  // Make a one-row-high sample array
  buffer = (*cinfo.mem->alloc_sarray)
		((j_common_ptr) &cinfo, JPOOL_IMAGE, row_stride, 1);


  // Step 6: while (scan lines remain to be read) 
  //           jpeg_read_scanlines(...);
  
  unsigned char* ptr = image;
 

 // Here we use the library's state variable cinfo.output_scanline as the
  // loop counter, so that we don't have to keep track ourselves.
 
  while (cinfo.output_scanline < cinfo.output_height) {
    // jpeg_read_scanlines expects an array of pointers to scanlines.
    // Here the array is only one element long, but you could ask for
    // more than one scanline at a time if that's more convenient.

    (void) jpeg_read_scanlines(&cinfo, buffer, 1);
    // Assume put_scanline_someplace wants a pointer and sample count. 
    //put_scanline_someplace(buffer[0], row_stride);
    //memcpy(buffer[0], image, row_stride); 
    memcpy(ptr, buffer[0], row_stride); 
    ptr += row_stride;
  }
  
  (void) jpeg_finish_decompress(&cinfo);	// Finish decompression
  jpeg_destroy_decompress(&cinfo);	// Release JPEG decompression object
  fclose(infile);

  //printf("+--------------------------------------\n\n");

  return image;

}







void write_JPEG_file(char filename[200], int quality, unsigned char* img, int width, int height)
{
  struct jpeg_compress_struct cinfo;
  struct jpeg_error_mgr jerr;
  FILE * outfile;			// target file 
  JSAMPROW row_pointer[1];		// pointer to JSAMPLE row[s] 
  int row_stride;			// physical row width in image buffer 

  //printf("    ________________________           \n");
  //printf("+--< Start: write_JPEG_file >----------\n");
  //printf("| Adress is:  %x\n",img); 
  //printf("| resolution: %i x %i\n",width, height); 

 cinfo.err = jpeg_std_error(&jerr);
 jpeg_create_compress(&cinfo);
 if ((outfile = fopen(filename, "wb")) == NULL) {
 	fprintf(stderr, "can't open %s\n", filename);
 	jpeg_destroy_compress(&cinfo);
 	return;
 	}
 jpeg_stdio_dest(&cinfo, outfile);
 cinfo.image_width  = width; 		// image width and height, in pixels
 cinfo.image_height = height;
 cinfo.input_components = 3;			// # of color components per pixel 
 cinfo.in_color_space = JCS_RGB; 		// colorspace of input image
 jpeg_set_defaults(&cinfo);
 jpeg_set_quality(&cinfo, quality, TRUE);
 jpeg_start_compress(&cinfo, TRUE);
 row_stride = width * 3;			// JSAMPLEs per row in image_buffer
 while (cinfo.next_scanline < cinfo.image_height) {
 	row_pointer[0] = & img[cinfo.next_scanline * row_stride];
 	(void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
 	}
 jpeg_finish_compress(&cinfo);
 fclose(outfile);
 jpeg_destroy_compress(&cinfo);

 //printf("| \n"); 
 //printf("+--------------------------------------\n\n");
}









//
// TGA support just for testing
//
//
void write_TGA_file(char filename[200], unsigned char* img, unsigned int width, unsigned int height )
{
 FILE *out;
 unsigned int t;
 unsigned int x, y;
 char ch;

  //printf("    _______________________            \n");
  //printf("+--< Start: write_TGA_file >-----------\n");
  //printf("| \n");
  //printf("| img adr: %x\n", img);
  //printf("| width:   %i\n", width);
  //printf("| height:  %i\n", height);
  //printf("|            \n");
  //printf("| file:    %s\n", filename);
  //printf("|--------------------------\n");


 //printf("\nwriting image");
 if((out = fopen(filename,"wb"))==NULL){
     printf("\n Error opening file %s to write!",filename);
     exit(0);  }

 for(t=1; t<=18; t++)		// TGA  HEADER 
 {
  ch=0;
  if(t==3) ch=2;
  if(t==13) ch= width & 255;	// x low
  if(t==14) ch= width>>8;	// x high
  if(t==15) ch= height & 255;	// y low
  if(t==16) ch= height>>8;	// y high
  if(t==17) ch= 24 ;		// 24-bit
  if(t==18) ch= 32 ;	
  fprintf(out,"%c",ch);
 }
 for(y=0; y<=height-1; y++)
  for(x=0; x<=width-1; x++)
  {
    fprintf(out,"%c",*(img+ ((x*3)+(y*width*3)+2)));
    fprintf(out,"%c",*(img+ ((x*3)+(y*width*3)+1)));
    fprintf(out,"%c",*(img+ ((x*3)+(y*width*3)+0)));
  }
 printf("| wrote file %s\n",filename);
 printf("+--------------------------------------\n\n");
 fclose(out); 
}

































